(* The interface to the automatically generated parser and lexer.  You
don't need to read this code. *)

structure Parser = struct

local

open Ast

(* connect up the generated lexer and parser *)
structure MiniMLLrVals = 
    MiniMLLrValsFun(structure Token = LrParser.Token)
structure MiniMLLex =
    MiniMLLexFun(structure Tokens = MiniMLLrVals.Tokens)
structure MiniMLParser =
    Join(structure ParserData = MiniMLLrVals.ParserData
	 structure Lex = MiniMLLex
	 structure LrParser = LrParser)

in

(* the ParseError exception is raised if there is a parse error *)
exception ParseError of string

(* parse the input string into a TopLevel AST, raising ParseError if
there was a syntax error *)
fun parse(inputString:string):TopLevel =
    let
        (* A hackish way of providing a pseudo-stream using the
           current string. *)
        val nextLine:string ref = ref inputString
        fun stringStream(_) =
            let val currentLine = !nextLine
	    in (nextLine := ""; currentLine) end
        val lexer = MiniMLParser.makeLexer(stringStream)
        fun print_error(msg:string,line:int,_) =
	    raise ParseError("line " ^ Int.toString(line) ^ ": " ^ msg)
        val (result,_) = MiniMLParser.parse(0, lexer, print_error, ())
            handle MiniMLLex.UserDeclarations.LexerError(msg:string) => 
	        raise ParseError(msg)
    in
        result
    end

end

end
