(* This is code to convert a Value to a printable string. *)

structure ValuePrint = struct

local
  open Value
  open AstPrint
in

fun valueToString(IntValue(i)) = Int.toString(i)
  | valueToString(StringValue(s)) = "\"" ^ s ^ "\""
  | valueToString(BoolValue(true)) = "true"
  | valueToString(BoolValue(false)) = "false"
  | valueToString(TupleValue(values)) = "(" ^ valuesToString(values) ^ ")"
  | valueToString(ListValue(values)) = "[" ^ valuesToString(values) ^ "]"
  | valueToString(FunValue{name,cases,env}) =
        "fn (* " ^ name ^ " *) " ^ valueCasesToString(cases)

and valueCasesToString([]) = ""
  | valueCasesToString([fcase]) = valueCaseToString(fcase)
  | valueCasesToString(fcase::fcase2::fcases) =
        valueCaseToString(fcase) ^ "\n | " ^ valueCasesToString(fcase2::fcases)

and valueCaseToString({formal,resType=NONE,body}) =
	patternToString(formal) ^ " => " ^ exprToString(body)
  | valueCaseToString({formal,resType,body}) =
	patternToString(formal) ^ " => " ^ exprToString(body) ^
	" (* " ^ typeOptionToString(resType) ^ " *)"

and valuesToString([]) = ""
  | valuesToString([value]) = valueToString(value)
  | valuesToString(value::value2::values) = 
	valueToString(value) ^ ", " ^ valuesToString(value2::values)

end

end
